﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Microsoft.TeamFoundation.WorkItemTracking.Client;
using System.Diagnostics;
using Microsoft.TeamFoundation.VersionControl.Client;
using Microsoft.TeamFoundation.Client;
using System.Configuration;

namespace MyTfsStoryBoard
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();

            var teamProjects = TfsRepository.GetAllProjects();

            DataSet ds = new DataSet();
            DataTable dt = new DataTable();
            DataColumn dc = new DataColumn();
            dc.ColumnName = "Team Project Name";
            dt.Columns.Add(dc);
            ds.Tables.Add(dt);

            foreach (var teamProject in teamProjects)
            {
                dt.Rows.Add(teamProject.Name);
            }

            dataGridView1.DataSource = ds.Tables[0];
            dataGridView1.Select();
        }

        private void tabPage2_Enter(object sender, EventArgs e)
        {
            var projCollections = TfsRepository.GetAllIterations();

            foreach (Project project in projCollections)
            {
                lstTeamProjects.Items.Add(project.Name);
            }
        }

        private void lstTeamProjects_SelectedIndexChanged(object sender, EventArgs e)
        {
            var selectedProject = ((ListBox)(sender)).SelectedItem.ToString();

            Project detailsOfTheSelectedProject = null;

            var projCollections = TfsRepository.GetAllIterations();

            foreach (Project project in projCollections)
            {
                if (!String.IsNullOrEmpty(selectedProject))
                {
                    if (project.Name.ToString().Contains(selectedProject))
                    {
                        detailsOfTheSelectedProject = project;
                        break;
                    }
                }
            }

            if (detailsOfTheSelectedProject != null)
            {
                lstIterations.Items.Clear();
                lstAreaPath.Items.Clear();
                lstQueries.Items.Clear();

                // Iterations
                foreach (Node node in detailsOfTheSelectedProject.IterationRootNodes)
                {
                    lstIterations.Items.Add(node.Name);

                    foreach (Node item in node.ChildNodes)
                    {
                        lstIterations.Items.Add(item.Name);
                    }
                }

                // Area Path
                foreach (Node area in detailsOfTheSelectedProject.AreaRootNodes)
                {
                    lstAreaPath.Items.Add(area.Name);

                    foreach (Node item in area.ChildNodes)
                    {
                        lstAreaPath.Items.Add(item.Name);
                    }
                }

                // Queries
                foreach (StoredQuery qi in detailsOfTheSelectedProject.StoredQueries)
                {
                    lstQueries.Items.Add(qi.Name);
                    Debug.Write(String.Format("Query Text: {0}{1}", qi.QueryText, Environment.NewLine));
                }
            }
        }

        private void lstQueries_SelectedIndexChanged(object sender, EventArgs e)
        {
            var selectedQuery = ((ListBox)(sender)).SelectedItem.ToString();

            var projCollections = TfsRepository.GetAllIterations();

            var tfs = TfsTeamProjectCollectionFactory.GetTeamProjectCollection(new Uri(ConfigurationManager.AppSettings["TfsUri"]));
            var wiStore = tfs.GetService<WorkItemStore>();

            Project detailsOfTheSelectedProject = null;

            foreach (Project project in projCollections)
            {
                if (!String.IsNullOrEmpty(lstTeamProjects.SelectedItem.ToString()))
                {
                    if (project.Name.ToString().Contains(lstTeamProjects.SelectedItem.ToString()))
                    {
                        detailsOfTheSelectedProject = project;
                        break;
                    }
                }
            }

            if (detailsOfTheSelectedProject != null)
            {
                tbQueryDetails.Text = string.Empty;
                lstWorkItems.Items.Clear();

                foreach (StoredQuery qi in detailsOfTheSelectedProject.StoredQueries)
                {
                    if (qi.Name.Contains(((ListBox)sender).SelectedItem.ToString()))
                    {
                        // Convert any parameters before converting the string to Query object, other wise will raise exception
                        tbQueryDetails.Text = detailsOfTheSelectedProject.Store.
                            GetQueryDefinition(qi.QueryGuid).QueryText.Replace("@project", String.Format("'{0}'", detailsOfTheSelectedProject.Name));

                        var query = new Query(wiStore, tbQueryDetails.Text);

                        if (query.IsLinkQuery)
                        {
                            var queryResults = query.RunLinkQuery();

                            lblQuery.Text = String.Format("Query Text Below | Query Type: {0}", "Linked Query");
                            label6.Text = String.Format("Total {0}", queryResults.Length);
                            foreach (WorkItemLinkInfo item in queryResults)
                            {
                                lstWorkItems.Items.Add(detailsOfTheSelectedProject.Store.GetWorkItem(item.TargetId).Id);
                            }
                        }
                        else
                        {
                            var queryResults = query.RunQuery();

                            lblQuery.Text = String.Format("Query Text Below | Query Type: {0}", "Flat Query");
                            label6.Text = String.Format("Total {0}", queryResults.Count);
                            foreach (WorkItem item in queryResults)
                            {
                                lstWorkItems.Items.Add(item.Id);
                            }
                        }
                    }
                }
            }
        }
    }
}
